LIBRARY		nessus.dll
EXETYPE WINDOWS
CODE		 PRELOAD MOVEABLE DISCARDABLE
DATA		 PRELOAD MOVEABLE MULTIPLE

EXPORTS
    arg_get                        @2
    arg_add_value                  @3
    arg_set_value                  @4
    arg_set_type                   @5
    arg_get_value                  @6
    arg_get_length                 @7
    arg_get_type                   @8
    arg_dup                        @9
    arg_dump                       @10
    arg_free                       @11
    arg_free_all                   @12
    ftp_log_in                     @13
    ftp_get_pasv_address           @14
    is_cgi_installed               @15
    open_sock_opt_hn               @16
    open_sock_tcp_hn               @17
    open_sock_tcp                  @18
    open_sock_udp                  @19
    open_sock_option               @20
    recv_line                      @21
    socket_close                   @22
    auth_printf                    @23
    auth_send                      @24
    auth_gets                      @25
    nessus_lib_version             @26
    plug_set_family                @27
    plug_set_dep                   @28
    plug_set_timeout               @29
    plug_get_family                @30
    plug_set_launch                @31
    plug_get_launch                @32
    plug_set_name                  @33
    plug_set_summary               @34
    plug_set_description           @35
    plug_set_copyright             @36
    plug_set_category              @37
    plug_set_attacklevel           @38
    plug_add_host                  @39
    host_add_port_proto            @40
    host_add_port                  @41
    host_add_port_udp              @42
    port_in_ports                  @43
    host_get_port_state_proto      @44
    host_get_port_state            @45
    host_get_port_state_udp        @46
    plug_get_hostname              @47
    plug_get_host_ip               @48
    proto_post_hole                @49
    post_hole                      @50
    post_hole_udp                  @51
    post_info                      @52
    post_info_udp                  @53
    proto_post_info                @54
    get_preference                 @57
    add_plugin_preference          @58
    get_plugin_preference          @59
    plug_set_key                   @60
    scanner_add_port               @61
    plug_get_key                   @62
    plug_get_host_open_port        @63
    nn_resolve                     @64
    qsort_compar                   @66
    getpts                         @67
    emalloc                        @68
    estrdup                        @69
    efree                          @70
    estrlen                        @71
    getopt                         @74
    getopt_long                    @75
    getopt_long_only               @76
    get_optarg                     @77
    get_opterr                     @78
    get_optind                     @79
    get_optopt                     @80
    inc_optind                     @81
    append_argv                    @82
    destroy_argv                   @83
    harg_create                    @84
    harg_dup                       @85
    harg_close                     @86
    harg_close_all                 @87
    harg_add                       @88
    harg_add_default               @89
    harg_remove                    @90
    harg_set_tvalue                @91
    harg_set_type                  @92
    harg_set_id                    @93
    harg_get_tvalue                @94
    harg_get_type                  @95
    harg_get_id                    @96
    harg_get_size                  @97
    harg_inx_key                   @98
    harg_dump                      @99
    harg_walk_init                @100
    harg_walk_next                @101
    harg_walk_stop                @102
    harg_do                       @103
    nessuslib_version		  @104
    banner_diff		 	  @105
